import os
from openai import OpenAI
import openai
from .utils import Video_Dataset
import json
import logging
from tenacity import retry, stop_after_attempt, wait_random_exponential


def eval(config, prompt,dimension):
    # 设置日志
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    file_handler = logging.FileHandler(config[f'log_path_{dimension}'])
    formatter = logging.Formatter('%(message)s')
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    client = OpenAI(
        api_key = config['GPT4o_API_KEY'],
        base_url = config['GPT4o_BASE_URL']
    )

    MODEL="gpt-4o-2024-08-06"
    models = ['cogvideox5b','gen3','kling','videocrafter2','pika','show1','lavie']

    dataset = Video_Dataset(data_dir=config['dataset_root_path'])
    results = {}    # 存储最终评分结果
    
    l1 = list(range(0, len(dataset)))
    for i in l1:
        logger.info(f'>>>>>>>>This is the {i} round>>>>>>>')
        data = dataset[i]
        results[i] = {}
        
        for model in models:
            try:
                modelname = model
                examplemodels = [x for x in models if x != modelname]
                frames = data['frames']
                prompten = data['prompt']

                messages=[
                    {
                        "role": "system", 
                        "content": prompt
                    },
                    {
                        "role": "user", 
                        "content":[
                            "According to **Important Notes** in system meassage, there are examples from other models.\n",
                            *[item for examplemodel in examplemodels for item in [
                                "This example is from model {} \n".format(examplemodels.index(examplemodel)+1),
                                {"type": "image_url", "image_url": {"url": f'data:image/jpg;base64,{frames[examplemodel][0]}', "detail": "low"}}
                            ]],              
                            
                            "These are the frames from the video you are evaluating. \n",
                            *map(lambda x: {"type": "image_url", 
                                "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}},frames[modelname]),    

                            "Assuming there are a video ' scoring 'x',provide your analysis and explanation in the output format as follows:\n"
                            "- video: x ,because ..."
                        ],
                    }
                ]

                # 调用API获取响应
                response = client.chat.completions.create(
                    model=MODEL,
                    messages=messages,
                    temperature=0
                )
                response_content = response.choices[0].message.content
                
                # 记录评估结果
                results[i][modelname] = response_content      # 可以根据需要提取分数部分
                
                logger.info(f'>>>>>>>Model {modelname} evaluation:\n{response_content}')

            except Exception as e:
                logger.error(f'Error evaluating model {modelname}: {str(e)}')
                results[i][modelname] = 'Error'

    # 返回符合主类期望的结构
    return {
        'score': results             # 评分结果
    }


import os
import importlib
from pathlib import Path
from .utils import save_json, load_json

class HABench(object):
    def __init__(self, full_info_dir, output_path, config_path):
        """
        Initialize VBench evaluator
        
        Args:
            full_info_dir (str): Path to the full info JSON file
            output_path (str): Directory to save evaluation results
            config_path (str): Path to configuration file
        """
        self.full_info_dir = full_info_dir  
        self.output_path = output_path
        self.config_path = config_path
        self.config = load_json(config_path)
        os.makedirs(self.output_path, exist_ok=True)

    def build_full_dimension_list(self):
        """Return list of all available evaluation dimensions"""
        return [
           "aesthetic_quality", "imaging_quality", "temporal_consistency", "motion_effects", 
           "color", "object_class", "scene", "action", "overall_consistency"
        ]

    def evaluate_dimension(self, dimension):
        """
        Evaluate a single dimension by importing and running its module
        
        Args:
            dimension (str): Name of the dimension to evaluate
            
        Returns:
            dict: Results of the evaluation
        """
        try:
            # Import the dimension's module
            VideoTextConsistency_dimensions = ['color', 'object_class', 'scene', 'action', 'overall_consistency']
            static_dimensions = ['aesthetic_quality', 'imaging_quality']
            dynamic_dimensions = ['temporal_consistency', 'motion_effects']

            if dimension in VideoTextConsistency_dimensions:
                from .VideoTextConsistency import eval
                from .prompt_dict import prompt
                results = eval(self.config, prompt[dimension],dimension)
            elif dimension in static_dimensions:
                from .staticquality import eval
                from .prompt_dict import prompt
                results = eval(self.config, prompt[dimension],dimension)
            elif dimension in dynamic_dimensions:
                from .dynamicquality import eval
                from .prompt_dict import prompt
                results = eval(self.config, prompt[dimension],dimension)
            else:
                raise ValueError(f"Unknown dimension: {dimension}")
            
                
            return results
            
        except Exception as e:
            print(f"Error evaluating {dimension}: {e}")
            return {'error': str(e)}

    def evaluate(self, videos_path, name, prompt_list=[], dimension_list=None, **kwargs):
        """
        Run evaluation on specified dimensions
        
        Args:
            videos_path (str): Path to video files
            name (str): Name for this evaluation run
            prompt_list (list): List of prompts
            dimension_list (list): List of dimensions to evaluate
            **kwargs: Additional arguments
        """
        # Initialize results dictionary
        VideoTextConsistency_dimensions = ['color', 'object_class', 'scene', 'action', 'overall_consistency']
        static_dimensions = ['aesthetic_quality', 'imaging_quality']
        dynamic_dimensions = ['temporal_consistency', 'motion_effects']

        
        # Use default dimension list if none provided
        if dimension_list is None:
            dimension_list = self.build_full_dimension_list()
            print(f'Using default dimension list: {dimension_list}')
        
        # Evaluate each dimension
        for dimension in dimension_list:
            print(f"Evaluating {dimension}...")
            results = self.evaluate_dimension(dimension)

            # 为每个维度创建输出目录
            dimension_output_dir = os.path.join(self.output_path, dimension)
            os.makedirs(dimension_output_dir, exist_ok=True)
            
            # 保存结果
            if dimension in VideoTextConsistency_dimensions:
                save_json(results['history'], os.path.join(dimension_output_dir, f'{name}_history_results.json'))
                save_json(results['updated_description'], os.path.join(dimension_output_dir, f'{name}_updated_description_results.json'))
                save_json(results['score'], os.path.join(dimension_output_dir, f'{name}_score_results.json'))
            else:
                save_json(results['score'], os.path.join(dimension_output_dir, f'{name}_score_results.json'))
        
        return results



import os
from typing import List, Dict, Any
from tqdm import tqdm
os.environ["TOKENIZERS_PARALLELISM"] = "false"
import re
import json
import yaml
import logging
import tenacity
from tenacity import retry, stop_after_attempt, wait_random_exponential
from openai import OpenAI
from .utils import Video_Dataset

class Agent():
    def __init__(self, agent, logger, prompt, config) -> None:
        """
        initialize agents, including functions that initiate chats and mllm calling
        agents: list of strings, each string is the name of an agent
        logger: logger object
        prompt: dict, system prompt, user prompt, and summary prompt
        config: dict, configurations
        """
        self.agent = agent
        self.logger = logger
        self.prompt = prompt
        self.config = config
        self.video_prompt=''
        self.history = []
        self.completion_tokens = 0
        self.prompt_tokens = 0

        # openai configurations
        self.api_key = self.config['GPT4o_mini_API_KEY']
        self.base_url = self.config['GPT4o_mini_BASE_URL']
        self.model = "gpt-4o-mini"

    # reset everything, avoid initializing too many classes
    def reset(self):
        self.video_prompt = ''
        self.history = []
        self.completion_tokens = 0
        self.prompt_tokens = 0

    @tenacity.retry(wait=tenacity.wait_exponential(max=60),
                    stop=tenacity.stop_after_attempt(5),
                    retry=tenacity.retry_if_exception_type(TypeError),
                    reraise=True)

        # call openai, input messages, get response
    @retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6))
    def call_oai(self, mess):
        client = OpenAI(api_key=self.api_key, base_url=self.base_url)
        response = client.chat.completions.create(
            model=self.model,
            messages=mess,
            temperature=0
        )
        completion_tokens = response.usage.completion_tokens
        prompt_tokens = response.usage.prompt_tokens

        self.completion_tokens += completion_tokens
        self.prompt_tokens += prompt_tokens

        return response.choices[0].message.content

    def prepare_message_text(self,agent):
        
        self.video_prompt = f'This is the text prompt:\n{self.video_prompt}'
        if not self.history:
            message_ = 'It\'s the first round.' + self.prompt[agent]+self.video_prompt
        # if not, get the history, and concat everything with proper format
        else:
            #prefix = ''
            # Create a header for the history section
            history_header = "There are the detailed video descriptions of the video:\n\n"
            history_content = '\n'.join(self.history)
            # Combine the header with the content
            history_ = history_header + "<video's description>\n" + history_content + "\n</video's description>"
            # Construct the final message with the provided prefix, history, and prompts
            message_ = self.video_prompt + '\n\n' + history_
        self.messages_text=[{"role": "system", "content": self.prompt[agent]},
            {"role": "user", "content": message_+'\n\nYour questions must be deisplayed in the format: \n<question>\n[your questions] or [I have no question]\n</question>.' + '\n\nDisplay the results in the specified Output Format'}]
        return self.messages_text


    def chat_for_one_round_text(self, agent) -> str:
        self.logger.info(f'-----Agent:{agent}-----')
        message = self.prepare_message_text(agent)
        response = self.call_oai(message)
        #self.history.append(response)
        self.logger.info(f'Round of {agent}:{response}')
        return response

# summarize the conversation and get the results

class Host():
    def __init__(self, name: str, logger, prompt, config, modelname,modelmessage,agents: List[Agent]):
        self.name = name
        self.agents = agents
        self.logger = logger
        self.prompt = prompt
        self.config = config
        self.modelname = modelname
        self.modelmessage = modelmessage
        self.video_prompt=" "
        self.frames=[]
        self.history = []
        self.qa_history = []
        self.description = ''
        self.api_key = self.config['GPT4o_API_KEY']
        self.base_url = self.config['GPT4o_BASE_URL']
        self.model = "gpt-4o-2024-08-06"
        self.completion_tokens = 0
        self.prompt_tokens = 0

    def reset(self):
        self.history = []
        self.qa_history = []
        self.frames=[]
        self.video_prompt = " "
        self.description = ''
        self.messages = []
        self.completion_tokens = 0
        self.prompt_tokens = 0

    def call_oai(self, mess):
        client = OpenAI(api_key=self.api_key, base_url=self.base_url)
        # client = AzureOpenAI(api_key=self.api_key, api_version=self.api_version, azure_endpoint=self.azure_endpoint)
        response = client.chat.completions.create(
            model=self.model,
            messages=mess,
            temperature=0
        )
        completion_tokens = response.usage.completion_tokens
        prompt_tokens = response.usage.prompt_tokens

        self.completion_tokens += completion_tokens
        self.prompt_tokens += prompt_tokens

        return response.choices[0].message.content

    def initial_result(self):
        self.video_prompt = f'This is the text prompt:\n{self.video_prompt}'
        message_ = "You must think following the 'Evaluation Steps' one by one.\n"
        self.messages = [{"role": "system", "content": self.prompt['gpt4o-system']},
                         {"role": "user", "content": [
                             {"type": "text", "text": message_},
                             {"type": "text", "text": '\n\nThese are the frames from the video generated by {}\n'.format(self.modelname)},
                             {"type": "text", "text": self.modelmessage},
                             *map(lambda x: {"type": "image_url",
                                             "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}},
                                  self.frames[self.modelname])
                         ]}]
 
        self.logger.info('==============initial response============')
        response = self.call_oai(self.messages)
        match = re.search(r'\[Video Description\]:\s*(.*)', response, re.DOTALL)
        if match:
            video_description = match.group(1).strip()
            self.history.append(f"This is the initial information: \n{video_description}\n\n")
        else:
            self.logger.warning("No video captured Description")
            video_description = "No video captured Description"
            self.history.append(f"This is the initial information: \n{video_description}\n\n")
        self.description = f'This is the video\'s initial description:\n<description>\n{response}\n</description>'
        return response

    def get_question(self, response):
        pattern = r"\[Your questions?\]:\s*(.*)"

        # 搜索并提取 [Your question]: 或 [Your questions]: 后的内容
        result = re.search(pattern, response, re.DOTALL)

        if result:
            # 提取 [Your question]: 或 [Your questions]: 后面的所有内容
            question = result.group(1).strip()
            return question
        else:
            # 如果没有匹配到 [Your question]: 或 [Your questions]:，返回 None
            return None

    def question(self) :

        # 1. 各智能体独立获得响应并添加到各自的历史记录中
        self.logger.info('==============agent qa_history============')
        # 2. 按顺序让每个智能体基于当前的历史记录做出回答
        #self.logger.info('==============reflect============')
        for i, agent in enumerate(self.agents):
            #combined_history = "\n".join(self.description)
            #print(combined_history)
            agent.history.append(self.description)
            response = agent.chat_for_one_round_text(agent.agent)
            question = self.get_question(response)
            self.qa_history.append(f"This is the question of the {agent.agent}: {question}\n")
            self.description = self.description+f'\nThis is a question of another assistant:\n<question-one>\n{question}\n</question-one>'

        return self.qa_history

    def answer(self):
        qa_history_header = '\n\nThere are the questions of two assistants:\n\n'
        qa_history_content = '\n'.join(self.qa_history)
        qa_history = qa_history_header + "<qa_history>\n" + qa_history_content + "\n</qa_history>\n"
        self.messages = [{"role": "system", "content":self.prompt['gpt4o-answer']},
                         {"role": "user", "content": [
                             {"type": "text", "text": self.video_prompt},
                             {"type": "text", "text": qa_history},
                             {"type": "text", "text": 'These are the frames from the video generated by {}\n'.format(self.modelname)},
                             # {"type": "text", "text": f"{self.modelmessage}; Please carefully observe whether the actions required by the text prompt appear in the video\n"},
                             {"type": "text", "text": f"{self.modelmessage}\n"},
                             *map(lambda x: {"type": "image_url",
                                             "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}},
                                  self.frames[self.modelname])
                         ]}]
        # print(messages)
        self.logger.info('==============qa_history============')
        response = self.call_oai(self.messages)
        match = re.search(r'\[Descriptions\]:(.*?)\[Answers\]', response, re.DOTALL)
        if match:
            description = match.group(1).strip()  # 提取匹配的内容并去除首尾空格
        else:
            print("No information found.")
        self.qa_history.append(f"\nThis is the answer of the questions: {response}\n\n")
        self.history.append(f"\nThis is the second information: {description}")
        #self.logger.info(f'>>>>>>>>>>qa_history:\n' + self.qa_history)
        return response
    
    def summarize_and_get_results(self) -> str:
        self.logger.info('==============final evaluation results============')
        #self.video_prompt = f'This is the text prompt:\n{self.video_prompt}\n'
        history_header = "There are the two informations:\n"
        history_content = '\n'.join(self.history)
        # Combine the header with the content
        history_ = history_header + "<history>\n" + history_content + "\n</history>"
        #history_ = 'This is the their evaluation history:\n\n<history>\n'.join(self.history)
        message_ = "You must think following the 'evaluation steps' one by one.\n\n" + history_ + '\n\n' + self.video_prompt+'\n\n'
        self.messages = [{"role": "system", "content": self.prompt['summer-system']},
                         {"role": "user", "content":[
                             {"type": "text", "text": message_},
                             {"type": "text", "text": f"{self.modelmessage}\n"},
                             *map(lambda x: {"type": "image_url",
                                             "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}},
                                  self.frames[self.modelname]),
                             {"type": "text", "text": 'The name of the AI medel is {}\n'.format(self.modelname)}
                         ]
                            }]
        #print(messages)
        response = self.call_oai(self.messages)
        return response

# 处理输出结果，按指定格式提取所需内容
def extract_content_from_result(final_result):
    valid_start_letters = ['l', 'p', 'k', 'v', 'g', 's', 'c']
    content_des = ""
    extracted_content = ""
    start_index = final_result.find("Updated Video Description")
    if start_index != -1:
        # 提取 "Updated Video Description" 后面的内容
        all_content = final_result[start_index + len("Updated Video Description"):].strip()

        # 找到第一个字母的位置
        first_letter_index = next((i for i, char in enumerate(all_content) if char.isalpha()), None)
        if first_letter_index is not None:
            all_content = all_content[first_letter_index:].strip()

        while True:
            eval_result_index = all_content.find("Evaluation Result")
            if eval_result_index != -1:
                content_des = all_content[:eval_result_index].strip()
                remaining_content = all_content[eval_result_index + len("Evaluation Result"):].strip()

                for a, char in enumerate(remaining_content):
                    if char.isalpha() and char.lower() in valid_start_letters:
                        content_score = remaining_content[a:].strip()
                        break
                else:
                    content_score = ""

                if content_score:
                    print(content_score)
                    break
                else:
                    all_content = all_content[eval_result_index + len("Evaluation Result"):].strip()
            else:
                print("未找到更多 'Evaluation Result' 部分")
                break
    else:
        print("未找到 'Updated Video Description' 部分")

    return content_des, content_score

def eval(config, prompt,dimension):
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    file_handler = logging.FileHandler(config[f'log_path_{dimension}'])
    formatter = logging.Formatter('%(message)s')
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    # load data
    dataset = Video_Dataset(data_dir=config['dataset_root_path'])
    #
    index = 0
    score = {}
    up_des = {}
    history = {}

    l1 = list(range(0, len(dataset)))
    for i in l1:
        data=dataset[i]

        score[i] = {}
        up_des[i] = {}
        history[i] = {}

        model2message = {
             'lavie': "5 frames from lavie.",
             'pika': "7 frames from pika.",
             'show1': "8 frames from show1.",
             'videocrafter2': "4 frames from videocrafter2.",
              'cogvideox5b': "12 frames from cogvideox5b. ",
             'kling': "10 frames from kling." ,
             'gen3': "20 frames from gen3.",
        }
        for key, value in model2message.items():
            modelname = key
            modelmessage = value
            agents = [Agent('Assistant-one', logger, prompt, config), Agent('Assistant-two', logger, prompt, config)]
            host = Host('Host', logger, prompt, config, modelname, modelmessage, agents)
            for agent in agents:
                agent.video_prompt = data['prompt']
            host.video_prompt = data['prompt']
            host.frames = data['frames']

            logger.info(f'>>>>>>>>This is the {i}_{modelname} round>>>>>>>')
            try:

                
                # 收集初始描述
                init_response = host.initial_result()
                logger.info(f'>>>>>>>>>>initial response:\n{init_response}')
                # 收集问答记录
                questions = host.question()
                logger.info(f'>>>>>>>>>>questions:\n{questions}')
                answers = host.answer()
                logger.info(f'>>>>>>>>>>answers:\n{answers}')
                # 基于描述和问答给出最终评分
                final_result = host.summarize_and_get_results()
                history[i][modelname] = {
                    'initial_response': init_response,
                    'qa_history': questions,
                    'final_result': final_result
                }
                logger.info(f'>>>>>>>the {i}_{modelname} round >>>>>>Discussion and judge:\n' + final_result)

                # 处理格式以写入json 
                content_des, content_score = extract_content_from_result(final_result)

                up_des[i][modelname] = content_des
                score[i][modelname] = content_score

            except Exception as e:
                logger.info('>>>>>>>>>>>Error occurred during conversation...')
                logger.info('Errormessage: ' + str(e))
                print(f"An error occurred: {e}")
                score[i][modelname] = 'Error'

            for agent in agents:
                agent.reset()
            host.reset()

    return {
        'history': history,  # 评估历史记录
        'updated_description': up_des,  # 视频最终描述
        'score': score         # 最终评分结果
    }




#!/usr/bin/env python
import os
import sys
import argparse
from datetime import datetime
from HAbench import HABench

def parse_args():
    parser = argparse.ArgumentParser(
        description='HABench - Human Preference Aligned Video Generation Benchmark',
        formatter_class=argparse.RawTextHelpFormatter
    )
    
    parser.add_argument(
        "--output_path",
        type=str,
        default='./evaluation_results/',
        help="output path to save the evaluation results"
    )
    
    parser.add_argument(
        "--config_path",
        type=str,
        default='./config.json',
        help="path to the config file"
    )
    
    parser.add_argument(
        "--videos_path",
        type=str,
        required=True,
        help="folder that contains the videos to evaluate"
    )
    
    parser.add_argument(
        "--dimension",
        nargs='+',
        choices=[
            'color',
            'object_class',
            'scene',
            'action',
            'overall_consistency',
            'imaging_quality',
            'aesthetic_quality',
            'temporal_consistency',
            'motion_effects'
        ],
        required=True,
        help="evaluation dimensions to use"
    )
    
    parser.add_argument(
        "--full_json_dir",
        type=str,
        default=None,
        help="path to save the full evaluation information json file"
    )

    return parser.parse_args()

def main():
    args = parse_args()
    
    # Create output directory if it doesn't exist
    os.makedirs(args.output_path, exist_ok=True)
    
    # Initialize benchmark
    benchmark = HABench()
    
    try:
        # Run evaluation
        results = benchmark.evaluate(
            videos_path=args.videos_path,
            dimensions=args.dimension,
            config_path=args.config_path,
            full_json_dir=args.full_json_dir
        )
        
        # Save results
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        output_file = os.path.join(args.output_path, f"results_{timestamp}.json")
        
        with open(output_file, 'w') as f:
            json.dump(results, f, indent=2)
            
        print(f"\nEvaluation completed successfully!")
        print(f"Results saved to: {output_file}")
        
    except Exception as e:
        print(f"\nError during evaluation: {str(e)}", file=sys.stderr)
        sys.exit(1)

if __name__ == "__main__":
    main() 


import os
import json
import cv2
import time
import base64
import numpy as np
from concurrent.futures import ThreadPoolExecutor

class Video_Dataset:
    def __init__(self, data_dir):
        self.data_dir = data_dir
        self.json_path = os.path.join(data_dir, 'human_anno/color.json')
        self.annotations = self.load_annotations()

    def load_annotations(self):
        with open(self.json_path, 'r') as f:
            return json.load(f)

    def process_video(self, datadir, videos_path, extract_frames_persecond=2, resize_fx=1, resize_fy=1):
        base64Frames = {
            "cogvideox5b": [],
            "kling": [],
            "gen3": [],
            "lavie": [],
            "pika": [],
            "show1": [],
            "videocrafter2": []
        }
        
        for key in base64Frames.keys():
            video = cv2.VideoCapture(os.path.join(datadir, videos_path[key]))

            if not video.isOpened():
                print(f"Error: Cannot open video file {datadir + videos_path[key]}")
                continue

            total_frames = int(video.get(cv2.CAP_PROP_FRAME_COUNT))
            fps = video.get(cv2.CAP_PROP_FPS)
            frames_to_skip = int(fps / extract_frames_persecond)

            curr_frame = 1
            end_frame = total_frames - 1
            
            # Loop through the video and extract frames at specified sampling rate
            while curr_frame < total_frames - 1:
                video.set(cv2.CAP_PROP_POS_FRAMES, curr_frame)
                success, frame = video.read()
                if not success:
                    break

                frame = cv2.resize(frame, None, fx=resize_fx, fy=resize_fx)
                _, buffer = cv2.imencode(".jpg", frame)
                base64Frames[key].append(base64.b64encode(buffer).decode("utf-8"))
                curr_frame += frames_to_skip

            # Get the last frame
            video.set(cv2.CAP_PROP_POS_FRAMES, end_frame)
            success, frame = video.read()
            if success:
                frame = cv2.resize(frame, None, fx=resize_fx, fy=resize_fx)
                _, buffer = cv2.imencode(".jpg", frame)
                base64Frames[key].append(base64.b64encode(buffer).decode("utf-8"))

            video.release()

        return base64Frames
    
    def process_video2gridview(self,datadir, videos_path, extract_frames_persecond=8):
        base64Frames = {"cogvideox5b": [], "kling": [], "gen3": [], "lavie": [], "pika": [], "show1": [], "videocrafter2": []}

        def process_video(key):
            frames = []
            video = cv2.VideoCapture(os.path.join(datadir, videos_path[key]))

            if not video.isOpened():
                print(f"Error: Cannot open video file {datadir+videos_path[key]}")
                return

            total_frames = int(video.get(cv2.CAP_PROP_FRAME_COUNT))
            fps = video.get(cv2.CAP_PROP_FPS)
            frames_to_skip = int(fps / extract_frames_persecond)
            curr_frame = 0

            while curr_frame < total_frames:
                video.set(cv2.CAP_PROP_POS_FRAMES, curr_frame)
                curr_frame += frames_to_skip
                success, frame = video.read()
                if not success:
                    break
                frames.append(frame)
                if len(frames) == extract_frames_persecond:
                    height, width, _ = frames[0].shape
                    grid_image = np.zeros((height, extract_frames_persecond * width, 3))

                    for j in range(extract_frames_persecond):
                        grid_image[0:height, j * width:(j + 1) * width] = frames[j]

                    _, buffer = cv2.imencode(".jpg", grid_image)
                    base64Frames[key].append(base64.b64encode(buffer).decode("utf-8"))
                    frames = []

            video.release()

        with ThreadPoolExecutor() as executor:
            executor.map(process_video, base64Frames.keys())

        return base64Frames


    def __len__(self):
        return len(self.annotations)

    def __getitem__(self, idx):
        annotation = self.annotations[idx]
        frames = self.process_video(self.data_dir, annotation['videos'], 2)
        grid_frames = self.process_video2gridview(self.data_dir, annotation['videos'], 8)
        
        return {
            'frames': frames,
            'grid_frames': grid_frames,
            'prompt': annotation['prompt_en']
        }

def save_json(data, path, indent=4):
    with open(path, 'w', encoding='utf-8') as f:
        json.dump(data, f, indent=indent)

def load_json(path):
    """
    Load a JSON file from the given file path.
    
    Parameters:
    - file_path (str): The path to the JSON file.
    
    Returns:
    - data (dict or list): The data loaded from the JSON file, which could be a dictionary or a list.
    """
    with open(path, 'r', encoding='utf-8') as f:
        return json.load(f)



import os
from openai import OpenAI
import openai
from .utils import Video_Dataset
import json
import logging
from tenacity import retry, stop_after_attempt, wait_random_exponential

@retry(wait=wait_random_exponential(min=1, max=60), stop=stop_after_attempt(6))
def call_api(client, messages, model):
    """调用 OpenAI API 的函数，包含重试机制"""
    response = client.chat.completions.create(
        model=model,
        messages=messages,
        temperature=0
    )
    return response.choices[0].message.content

def eval(config, prompt, dimension):
    """
    Evaluate videos using OpenAI API
    Args:
        config: configuration dictionary
        prompt: prompt template
        dimension: evaluation dimension name
    Returns:
        dict: containing evaluation scores
    """
    # 设置日志
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    file_handler = logging.FileHandler(config[f'log_path_{dimension}'])
    formatter = logging.Formatter('%(message)s')
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    client = OpenAI(
        api_key = config['GPT4o_API_KEY'],
        base_url = config['GPT4o_BASE_URL']
    )
    MODEL = "gpt-4o-2024-08-06"

    # 初始化结果字典
    results = {}
    
    # 加载数据集
    dataset = Video_Dataset(data_dir=config['dataset_root_path'])
    
    # 处理每组视频
    l1 = list(range(0, len(dataset)))
    for i in l1:
        try:
            logger.info(f'Processing video {i}...')
            data = dataset[i]
            frames = data['frames']
            prompten = data['prompt']
            
            # 构建消息
            messages = [
                {
                    "role": "system",
                    "content": prompt
                },
                {
                    "role": "user", "content": [
                        "These are the frames from the video.The prompt is '{}'.".format(prompten),
                        "12 frames from cogvideox5b \n ", 
                        *map(lambda x: {"type": "image_url", 
                                        "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}}, frames['cogvideox5b']),
                        "10 frames from kling \n ", 
                        *map(lambda x: {"type": "image_url", 
                                        "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}}, frames['kling']),
                        "20 frames from gen3 \n ", 
                        *map(lambda x: {"type": "image_url", 
                                        "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}}, frames['gen3']),
                        " 4 frames from videocrafter2 \n ",
                        *map(lambda x: {"type": "image_url", 
                                        "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}}, frames['videocrafter2']),   
                        "\n 7 frames from pika \n",
                        *map(lambda x: {"type": "image_url", 
                                        "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}}, frames['pika']),
                        "\n 8 frames from show1\n ",
                        *map(lambda x: {"type": "image_url", 
                                        "image_url": {"url":    f'data:image/jpg;base64,{x}', "detail": "low"}}, frames['show1']),                             
                        "\n5 frames from lavie\n ",
                        *map(lambda x: {"type": "image_url", 
                                        "image_url": {"url": f'data:image/jpg;base64,{x}', "detail": "low"}},frames['lavie']),
                                                                ], 
            }
            ]

            # 调用 API 并获取结果
            response = call_api(client, messages, MODEL)
            results[str(i)] = response
            
            logger.info(f'Successfully evaluated video {i}')
            logger.debug(f'Response for video {i}: {response}')
            
        except Exception as e:
            logger.error(f'Error processing video {i}: {str(e)}')
            results[str(i)] = f'Error: {str(e)}'

    return {
        'score': results
    }


from .prompt.overall_consistency import *
from .prompt.color import *
from .prompt.object_class import *
from .prompt.scene import *
from .prompt.action import *
from .prompt.PromptTemplate4GPTeval import Prompt4ImagingQuality
from .prompt.PromptTemplate4GPTeval import Prompt4AestheticQuality
from .prompt.PromptTemplate4GPTeval import Prompt4TemporalConsistency
from .prompt.PromptTemplate4GPTeval import Prompt4MotionEffects


prompt = {
    "overall_consistency": overall_consistency_prompt,
    "color": color_prompt,
    "object_class": object_class_prompt,
    "scene": scene_prompt,
    "action": action_prompt,
    "imaging_quality": Prompt4ImagingQuality,
    "aesthetic_quality": Prompt4AestheticQuality,
    "temporal_consistency": Prompt4TemporalConsistency,
    "motion_effects": Prompt4MotionEffects
 }


#!/usr/bin/env python

from setuptools import find_packages, setup
import os

def fetch_readme():
    with open('README.md', encoding='utf-8') as f:
        text = f.read()
    return text

def fetch_requirements():
    filename = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'requirements.txt')
    with open(filename, 'r') as f:
        envs = [line.rstrip('\n') for line in f.readlines() if '@' not in line]
    return envs

install_requires = fetch_requirements()
setup(name='HAbench',
      version='0.1.0',
      description='Human Preference Aligned Video Generation Benchmark',
      long_description=fetch_readme(),
      long_description_content_type='text/markdown',
      project_urls={
          'Source': 'https://github.com/HA-Video-Bench/HA-Video-Bench',
      },
      entry_points={
          'console_scripts': ['HAbench=HAbench.cli.evaluate:main']
      },
      install_requires=install_requires,
      packages=find_packages(),
      include_package_data=True,
      license='Apache Software License 2.0',
)
